

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
    *                                                                   *
    *   Some DSP 0.2                                                    *
    *                                                                   *
    *   Copyright  2005   Sebastian Pipping <webmaster@hartwork.org>   *
    *                                                                   *
    *   --> http://www.hartwork.org                                     *
    *                                                                   *
    *                                                                   *
    *   This source code is released under LGPL.                        *
    *   See LGPL.txt for details.                                       *
    *                                                                   *
    \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


#include "mod_delay.h"


void config_delay( struct winampDSPModule * this_mod );
int init_delay( struct winampDSPModule * this_mod );
int apply_delay( struct winampDSPModule * this_mod, short int * samples, int numsamples, int bps, int nch, int srate );
void quit_delay( struct winampDSPModule * this_mod );


#define DELAY_TIME	60		// ms
#define DELAY_PERC	0.33	// 0..1

short int * hist_data = NULL;	// History ring buffer
int hist_len = 0;	// Length of ring buffer
int hist_end = 0;	// Offset for next write
int hist_written = 0;	// Entries already filled


////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
winampDSPModule mod_delay =
{
	"Single Delay",
	NULL,			// hwndParent
	NULL,			// hDllInstance
	config_delay,
	init_delay,
	apply_delay,
	quit_delay
};


////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
void config_delay( struct winampDSPModule * this_mod )
{
	MessageBox(	this_mod->hwndParent,
				"Single Delay\n"
				"\n"
				"Copyright  2005 Sebastian Pipping    \n"
				"--> http://www.hartwork.org",
				"About",
				MB_ICONINFORMATION );
}


////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
int init_delay( struct winampDSPModule * this_mod )
{
	hist_len = ( 2 * 44100 * DELAY_TIME / 1000 );	// Just enough
	hist_data = new short int[ hist_len ];

	return 0;	// Fine
}


////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
inline short int boost_sample( double sample )
{
	static const double tan1 = tan( 1.0 );
	return ( short int )( 32768.0 * atan( sample / 32768.0 * tan1 ) );
}


////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
int apply_delay( struct winampDSPModule * this_mod, short int * samples, int numsamples, int bps, int nch, int srate )
{
	const int block_len = numsamples * nch;
	static int i;	// Same each time == less mem


	// Enough history?
	if( hist_written >= ( nch * srate * DELAY_TIME / 1000 ) )
	{
		// Apply echo
		int copy_beg = hist_end - ( nch * srate * DELAY_TIME / 1000 );
		while( copy_beg < 0 )
		{
			copy_beg += hist_len;
		}
		
		if( copy_beg + block_len <= hist_len )
		{
			// At once
			for( i = 0; i < block_len; i++ )
			{
				samples[ i ] = boost_sample( samples[ i ] * ( 1.0 - DELAY_PERC ) + hist_data[ copy_beg + i ] * ( DELAY_PERC ) );
			}
		}
		else
		{
			// First
			const int first = hist_len - copy_beg;
			for( i = 0; i < first; i++ )
			{
				samples[ i ] = boost_sample( samples[ i ] * ( 1.0 - DELAY_PERC ) + hist_data[ copy_beg + i ] * ( DELAY_PERC ) );
			}
	
			// Second
			const int second = block_len - first;
			for( i = first; i < block_len; i++ )
			{
				samples[ i ] = boost_sample( samples[ i ] * ( 1.0 - DELAY_PERC ) + hist_data[ i - first ] * ( DELAY_PERC ) );
			}
		}
	}
	else
	{
		hist_written += block_len;
	}


	// Copy modified samples to history
	const int space_left = hist_len - hist_end;
	if( space_left >= block_len )
	{
		// At once
		memcpy( hist_data + hist_end, samples, sizeof( short int ) * block_len );
		hist_end += block_len;
	}
	else
	{
		// First
		const int first = space_left;
		memcpy( hist_data + hist_end, samples, sizeof( short int ) * first );

		// Second
		const int second = block_len - space_left;
		memcpy( hist_data, samples + first, sizeof( short int ) * second );


		hist_end = second;
	}


	return numsamples;
}


////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
void quit_delay( struct winampDSPModule * this_mod )
{
	delete [] hist_data;
}
